use WebSecurity;
GO
-- Drop stored procedure if it already exists
IF EXISTS (
  SELECT * 
    FROM INFORMATION_SCHEMA.ROUTINES 
   WHERE SPECIFIC_CATALOG = N'WebSecurity' 
	 AND SPECIFIC_SCHEMA = N'dbo'
     AND SPECIFIC_NAME = N'PasswordQuestion_Search' 
)
   DROP PROCEDURE dbo.PasswordQuestion_Search
GO
create procedure PasswordQuestion_Search
    @PageSize int ,
    @PageNumber int, 
    @SortColumn nVarChar (100) ,
    @SortType nVarChar (4),
    @TotalRows int = null OUTPUT, 
@PasswordQuestionDesc NVarChar(250) = N'%' , 
	@RowsAffected int = null OUTPUT 
--*************************************************************
--Created By: trooks
--Name: .dbo_PasswordQuestion_Search
--Table: PasswordQuestion
--Database: WebSecurity
--Created Date: 01/21/2013 00:00:00
--*************************************************************

as
	Begin Try
        Declare @StartIndex int = (@PageNumber - 1) * @PageSize + 1
        Declare @EndingIndex int = (@PageSize - 1) * @PageNumber + @PageNumber 
        Set @TotalRows = (Select Count(*) from dbo.PasswordQuestion)
        IF @SortType = 'asc'
				BEGIN
					WITH tempPasswordQuestion  AS
					(
                        SELECT	ROW_NUMBER() OVER (ORDER BY @SortColumn  asc ) AS RowNumber,
		PasswordQuestionID, 
		PasswordQuestionDesc, 
		TS, 
		CreatedBy, 
		CreatedDt, 
		ModifiedBy, 
		ModifiedDt 
        
		From [dbo].PasswordQuestion 
Where PasswordQuestionDesc Like '%' + @PasswordQuestionDesc + '%'  
		)
                    SELECT	RowNumber, 
		PasswordQuestionID, 
		PasswordQuestionDesc, 
		TS, 
		CreatedBy, 
		CreatedDt, 
		ModifiedBy, 
		ModifiedDt 
                    FROM	tempPasswordQuestion
					WHERE	RowNumber BETWEEN @StartIndex AND @EndingIndex
					ORDER BY RowNumber ASC;
                    
		        END
			ELSE
                BEGIN
					WITH tempPasswordQuestion  AS
					(
                        SELECT	ROW_NUMBER() OVER (ORDER BY @SortColumn  desc ) AS RowNumber,
		PasswordQuestionID, 
		PasswordQuestionDesc, 
		TS, 
		CreatedBy, 
		CreatedDt, 
		ModifiedBy, 
		ModifiedDt 
        
		From [dbo].PasswordQuestion 
Where PasswordQuestionDesc Like '%' + @PasswordQuestionDesc + '%'  
		)
                    SELECT	RowNumber, 
		PasswordQuestionID, 
		PasswordQuestionDesc, 
		TS, 
		CreatedBy, 
		CreatedDt, 
		ModifiedBy, 
		ModifiedDt 
                    FROM	tempPasswordQuestion
					WHERE	RowNumber BETWEEN @StartIndex AND @EndingIndex
					ORDER BY RowNumber ASC;
                    
		        END
		Set @RowsAffected = @@ROWCOUNT;
		return 0;	
	END TRY
	BEGIN CATCH
		Set @RowsAffected = 0;
		EXEC dbo.uspLogError;
		return -1;
	END CATCH
	
GO
--GRANT EXECUTE ON [WebSecurity].[dbo].[PasswordQuestion_Search] to PORTAL_WebSecurity