
use WebSecurity;
GO
-- Drop stored procedure if it already exists
IF EXISTS (
  SELECT * 
    FROM INFORMATION_SCHEMA.ROUTINES 
   WHERE SPECIFIC_CATALOG = N'WebSecurity' 
	 AND SPECIFIC_SCHEMA = N'dbo'
     AND SPECIFIC_NAME = N'UserProfile_GetAll' 
)
   DROP PROCEDURE dbo.UserProfile_GetAll
GO
create procedure dbo.UserProfile_GetAll
    @PageSize int ,
    @PageNumber int, 
    @SortColumn nVarChar (100) ,
    @SortType nVarChar (4),
    @TotalRows int = null OUTPUT, 
	@RowsAffected int = null OUTPUT 
	
--*************************************************************
--Created By: trooks
--Name: dbo_UserProfile_GetAll
--Table: UserProfile
-- Schema: dbo
--Database: WebSecurity
--Created Date: 01/21/2013 00:00:00
--*************************************************************

as
	Begin Try
        Declare @StartIndex int = (@PageNumber - 1) * @PageSize + 1
        Declare @EndingIndex int = (@PageSize - 1) * @PageNumber + @PageNumber 
        Set @TotalRows = (Select Count(*) from dbo.UserProfile)
        IF @SortType = 'asc'
				BEGIN
					WITH tempUserProfile  AS
					(
                        SELECT	ROW_NUMBER() OVER (ORDER BY @SortColumn  asc ) AS RowNumber,
						UserId, 
						UserName, 
						TS, 
						CreatedBy, 
						CreatedDt, 
						ModifiedBy, 
						ModifiedDt, 
						IsActive, 
						FirstName, 
						LastName, 
						Address, 
						City, 
						Region, 
						PostalCode, 
						OfficePhone, 
						CellPhone, 
						Title 
      
                    FROM dbo.UserProfile
                    )
                    SELECT	RowNumber, 
					UserId, 
						UserName, 
						TS, 
						CreatedBy, 
						CreatedDt, 
						ModifiedBy, 
						ModifiedDt, 
						IsActive, 
						FirstName, 
						LastName, 
						Address, 
						City, 
						Region, 
						PostalCode, 
						OfficePhone, 
						CellPhone, 
						Title 
 
					FROM	tempUserProfile
					WHERE	RowNumber BETWEEN @StartIndex AND @EndingIndex
					ORDER BY RowNumber ASC;
                    
		        END
			ELSE
		        BEGIN
					WITH tempUserProfile  AS
					(
                        SELECT	ROW_NUMBER() OVER (ORDER BY @SortColumn  desc ) AS RowNumber,
						UserId, 
						UserName, 
						TS, 
						CreatedBy, 
						CreatedDt, 
						ModifiedBy, 
						ModifiedDt, 
						IsActive, 
						FirstName, 
						LastName, 
						Address, 
						City, 
						Region, 
						PostalCode, 
						OfficePhone, 
						CellPhone, 
						Title 
      
                    FROM dbo.UserProfile
                    )
                    SELECT	
					UserId, 
						UserName, 
						TS, 
						CreatedBy, 
						CreatedDt, 
						ModifiedBy, 
						ModifiedDt, 
						IsActive, 
						FirstName, 
						LastName, 
						Address, 
						City, 
						Region, 
						PostalCode, 
						OfficePhone, 
						CellPhone, 
						Title 
 
					FROM	tempUserProfile
					WHERE	RowNumber BETWEEN @StartIndex AND @EndingIndex
					ORDER BY RowNumber ASC;
                    
		        END
		

		Set @RowsAffected = @@ROWCOUNT;
		return 0;	
	END TRY
	BEGIN CATCH
		Set @RowsAffected = 0;
		EXEC dbo.uspLogError;
		return -1;
	END CATCH
	
GO
GRANT EXECUTE ON [WebSecurity].[dbo].[UserProfile_GetAll] to PORTAL_WebSecurity
GO