﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using T2.WebSecurity.ASPMVC.Controllers;
using T2.WebSecurity.ASPMVC.Entity;
using Microsoft.QualityTools.Testing.Fakes;
using T2.WebSecurity.ASPMVC.Entity.Interface;
using System.Collections.Generic;
using System.Net.Http;

namespace T2.WebSecurity.UI.MVC.Tests.Controllers
{
    [TestClass]
    public class ApplicationUserController
    {
        private static List<IApplicationUser_ui> retVal;

        [TestMethod]
        public void ApplicationUser_1_GetAll()
        {
            string testUserName = "trooks";
            string testPassword = "duminda123";
            bool testRememberMe = false;

            ApplicationUserAPIController controller = new ApplicationUserAPIController();
            ApplicationUser_ui param = new ApplicationUser_ui();
            param.Action = "GetAll";
            param.PageNumber = 1;
            param.PageSize = 100;
            param.SortColumn = "ApplicationUserID";
            param.SortType = "asc";

            using (ShimsContext.Create())
            {
                System.Web.Security.Fakes.ShimMembership.GetUser = () =>
                {
                    return new System.Web.Security.MembershipUser(
                        "T2MembershipProvider", "trooks", 1, "dumindapium@gmail.com", "car",
                        "none", true, false, Convert.ToDateTime("1/1/2013"), Convert.ToDateTime("1/1/2013"),
                        Convert.ToDateTime("1/10/2013"), Convert.ToDateTime("1/1/2013"), Convert.ToDateTime("1/1/2013"));
                };
                System.Web.Security.Fakes.ShimFormsAuthentication.SetAuthCookieStringBoolean = (userName, rememberMe) =>
                {
                    Assert.AreEqual(testUserName, userName);
                    Assert.AreEqual(testRememberMe, rememberMe);
                };
                System.Web.Security.Fakes.ShimRoles.GetRolesForUser = () =>
                {
                    return new string[] { "super admin" };
                };
                WebMatrix.WebData.WebSecurity.Login(testUserName, testPassword, false);

                retVal = controller.GetApplicationUser(param.Action, param.Caller, param.SortColumn, param.SortType, param.Role, param.PageNumber, param.PageSize);
                Assert.IsNotNull(retVal);
                Assert.IsTrue(retVal[0].ApplicationUserID > 0);
            }
        }

        [TestMethod]
        public void ApplicationUser_2_Add()
        {
            string testUserName = "trooks";
            string testPassword = "duminda123";
            bool testRememberMe = false;

            ApplicationUserAPIController controller = new ApplicationUserAPIController();
            ApplicationUser_ui param = new ApplicationUser_ui();
            param.Action = "Add";
            param.ApplicationID = 1;
            param.UserId = 1;

            using (ShimsContext.Create())
            {
                System.Web.Security.Fakes.ShimMembership.GetUser = () =>
                {
                    return new System.Web.Security.MembershipUser(
                        "T2MembershipProvider", "trooks", 1, "dumindapium@gmail.com", "car",
                        "none", true, false, Convert.ToDateTime("1/1/2013"), Convert.ToDateTime("1/1/2013"),
                        Convert.ToDateTime("1/10/2013"), Convert.ToDateTime("1/1/2013"), Convert.ToDateTime("1/1/2013"));
                };
                System.Web.Security.Fakes.ShimFormsAuthentication.SetAuthCookieStringBoolean = (userName, rememberMe) =>
                {
                    Assert.AreEqual(testUserName, userName);
                    Assert.AreEqual(testRememberMe, rememberMe);
                };
                System.Web.Security.Fakes.ShimRoles.GetRolesForUser = () =>
                {
                    return new string[] { "super admin" };
                };
                WebMatrix.WebData.WebSecurity.Login(testUserName, testPassword, false);

                HttpResponseMessage msg = controller.PostApplicationUser(param);
                Assert.IsNotNull(msg);
                Assert.IsTrue(msg.StatusCode == System.Net.HttpStatusCode.OK || msg.StatusCode == System.Net.HttpStatusCode.NotAcceptable);
            }
        }

        [TestMethod]
        public void ApplicationUser_2_Edit()
        {
            string testUserName = "trooks";
            string testPassword = "duminda123";
            bool testRememberMe = false;

            ApplicationUserAPIController controller = new ApplicationUserAPIController();
            ApplicationUser_ui param = new ApplicationUser_ui();
            param.Action = "Update";
            param.ApplicationUserID = 3018;
            param.TS = "0-0-0-0-0-0-226-179";
            param.ApplicationID = 1021;
            param.UserId = 2027;
            param.CreatedBy = "duminda";
            param.CreatedDt = Convert.ToDateTime("2013-08-29 16:00:23.987");
            param.ModifiedBy = "trooks";
            param.ModifiedDt = Convert.ToDateTime("2013-08-29 16:06:39.890");

            using (ShimsContext.Create())
            {
                System.Web.Security.Fakes.ShimMembership.GetUser = () =>
                {
                    return new System.Web.Security.MembershipUser(
                        "T2MembershipProvider", "trooks", 1, "dumindapium@gmail.com", "car",
                        "none", true, false, Convert.ToDateTime("1/1/2013"), Convert.ToDateTime("1/1/2013"),
                        Convert.ToDateTime("1/10/2013"), Convert.ToDateTime("1/1/2013"), Convert.ToDateTime("1/1/2013"));
                };
                System.Web.Security.Fakes.ShimFormsAuthentication.SetAuthCookieStringBoolean = (userName, rememberMe) =>
                {
                    Assert.AreEqual(testUserName, userName);
                    Assert.AreEqual(testRememberMe, rememberMe);
                };
                System.Web.Security.Fakes.ShimRoles.GetRolesForUser = () =>
                {
                    return new string[] { "super admin" };
                };
                WebMatrix.WebData.WebSecurity.Login(testUserName, testPassword, false);

                HttpResponseMessage msg = controller.PostApplicationUser(param);
                Assert.IsNotNull(msg);
                Assert.IsTrue(msg.StatusCode == System.Net.HttpStatusCode.OK || msg.StatusCode == System.Net.HttpStatusCode.NotAcceptable);
            }
        }

        [TestMethod]
        public void ApplicationUser_2_Delete()
        {
            string testUserName = "trooks";
            string testPassword = "duminda123";
            bool testRememberMe = false;

            ApplicationUserAPIController controller = new ApplicationUserAPIController();
            ApplicationUser_ui param = new ApplicationUser_ui();
            param.Action = "Remove";
            param.ApplicationUserID = 3018;
            param.TS = "0-0-0-0-0-0-226-179";
            param.ApplicationID = 1021;
            param.UserId = 2027;
            param.CreatedBy = "duminda";
            param.CreatedDt = Convert.ToDateTime("2013-08-29 16:00:23.987");
            param.ModifiedBy = "trooks";
            param.ModifiedDt = Convert.ToDateTime("2013-08-29 16:06:39.890");

            using (ShimsContext.Create())
            {
                System.Web.Security.Fakes.ShimMembership.GetUser = () =>
                {
                    return new System.Web.Security.MembershipUser(
                        "T2MembershipProvider", "trooks", 1, "dumindapium@gmail.com", "car",
                        "none", true, false, Convert.ToDateTime("1/1/2013"), Convert.ToDateTime("1/1/2013"),
                        Convert.ToDateTime("1/10/2013"), Convert.ToDateTime("1/1/2013"), Convert.ToDateTime("1/1/2013"));
                };
                System.Web.Security.Fakes.ShimFormsAuthentication.SetAuthCookieStringBoolean = (userName, rememberMe) =>
                {
                    Assert.AreEqual(testUserName, userName);
                    Assert.AreEqual(testRememberMe, rememberMe);
                };
                System.Web.Security.Fakes.ShimRoles.GetRolesForUser = () =>
                {
                    return new string[] { "super admin" };
                };
                WebMatrix.WebData.WebSecurity.Login(testUserName, testPassword, false);

                HttpResponseMessage msg = controller.PostApplicationUser(param);
                Assert.IsNotNull(msg);
                Assert.IsTrue(msg.StatusCode == System.Net.HttpStatusCode.OK || msg.StatusCode == System.Net.HttpStatusCode.NotAcceptable);
            }
        }
    }
}
